# Generated by roxygen2: do not edit by hand

S3method(coef,LMMsolve)
S3method(deviance,LMMsolve)
S3method(fitted,LMMsolve)
S3method(logLik,LMMsolve)
S3method(predict,LMMsolve)
S3method(print,summary.LMMsolve)
S3method(residuals,LMMsolve)
S3method(summary,LMMsolve)
export(LMMsolve)
export(LMMsolveObject)
export(as.ginverse)
export(diagnosticsMME)
export(displayMME)
export(effDim)
export(getHeritability)
export(mLogLik)
export(makeGrid)
export(multinomial)
export(obtainSmoothTrend)
export(spl1D)
export(spl2D)
export(spl3D)
importFrom(Rcpp,sourceCpp)
importFrom(methods,new)
importFrom(stats,.getXlevels)
importFrom(stats,aggregate)
importFrom(stats,as.formula)
importFrom(stats,coef)
importFrom(stats,contrasts)
importFrom(stats,formula)
importFrom(stats,gaussian)
importFrom(stats,logLik)
importFrom(stats,model.frame)
importFrom(stats,model.matrix)
importFrom(stats,model.response)
importFrom(stats,na.pass)
importFrom(stats,setNames)
importFrom(stats,terms)
importFrom(stats,terms.formula)
importFrom(stats,update)
importFrom(stats,var)
importFrom(utils,hasName)
useDynLib(LMMsolver, .registration = TRUE)
